import 'dart:convert';
import 'dart:io';

import 'package:hqsoft_core/core_base/core_base.dart';
import 'package:hqsoft_core/data_base/data_base.dart';
import 'package:http/io_client.dart';


enum ServiceErrorType { Error, SocketException, TimeoutException, IOException }
class Service {
  final int serviceTimeOut = 20;

  static final RegExp _jsonMessageRegExp = RegExp(r'"message":');

  static isHasJsonMessage(String message) {
    return _jsonMessageRegExp.hasMatch(message);
  }

  Future<String> getServiceAPI() async {
    String apiUrl =
        await serviceCountryPreferences.getPreferredServiceCountryAPIURL();
    return apiUrl;
  }

  IOClient createHttpClient() {
    String certSSL = "";
    HttpClient httpClient = new HttpClient()
      ..badCertificateCallback =
          ((X509Certificate cert, String host, int port) {
        if (certSSL == "")
          return true;
        else {
          // ignore: unrelated_type_equality_checks
          if (cert == certSSL)
            return true;
          else
            return false;
        }
      });
    IOClient ioClient = new IOClient(httpClient);
    return ioClient;
  }

  Future<Map<String, String>> createHeaderUnAuthorization() async {
    var headers;
    var headersContentType = "application/json";
    var langID = await allTranslations.currentLanguage;
    headers = {'Content-type': '$headersContentType', 'LangID': '$langID'};
    return headers;
  }

  Future<Map<String, String>> createHeaderAuthorization(
      {String? tokenAuthor}) async {
    var headers;
    var headersContentType = "application/json";
    var langID = await allTranslations.currentLanguage;
    var typeAuthorization = "Bearer ";
    var token = tokenAuthor ?? await tokenPreferences.getPreferredToken();
    headers = {
      'Content-type': '$headersContentType',
      'LangID': '$langID',
      'Authorization': '$typeAuthorization$token'
    };
    return headers;
  }

  Future<Map<String, String>> createHeaderAuthorizationRequiredToken(
      String? tokenAuthor) async {
    var headers;
    var headersContentType = "application/json";
    var langID = await allTranslations.currentLanguage;
    var typeAuthorization = "Bearer ";
    var token = tokenAuthor ?? await tokenPreferences.getPreferredToken();
    headers = {
      'Content-type': '$headersContentType',
      'LangID': '$langID',
      'Authorization': '$typeAuthorization$token'
    };
    return headers;
  }

  Future<Map<String, String>> createHeaderAuthorizationHQSoft() async {
    var headers;
    var headersContentType = "application/json";
    var langID = await allTranslations.currentLanguage;
    headers = {
      'Content-type': '$headersContentType',
      'LangID': '$langID',
      'Authorization': 'Es+rcgytCHVjrFz74kAngg@@'
    };
    return headers;
  }

  Future<Map<String, String>> createHeaderAuthorizationGuid(
      String guid) async {
    var headers;
    var headersContentType = "application/json";
    var langID = await allTranslations.currentLanguage;
    var typeAuthorization = "Bearer ";
    var token = await tokenPreferences.getPreferredToken();
    headers = {
      'x-requestid': '$guid',
      'Content-type': '$headersContentType',
      'LangID': '$langID',
      'Authorization': '$typeAuthorization$token'
    };
    return headers;
  }

  Future<Map<String, String>> createHeaderProductSearchAzure() async {
    var headers;
    var headersContentType = "application/json";
    var apiKey = 'B53C136C0946A3175007BA0E73FDEF2D';
    headers = {'Content-type': '$headersContentType', 'api-key': '$apiKey'};
    return headers;
  }

  static String parseStringBody(String body) {
    String resultToken = "";
    List<String> result = body.split("\"");
    if (result.length > 1)
      resultToken = result[1];
    else
      resultToken = body;
    return resultToken;
  }

  String handleErrorToString(int statusCode, String error) {
    String mess = "";
    if (isResponseErrorNetwork(statusCode))
      mess = allTranslations.text("mess_network_failed");
    else {
      if (isHasJsonMessage(error)) {
        mess = JsonMessage.fromJson(jsonDecode(error)).message.toString();
      } else
        mess = error;
    }
    return mess;
  }

  bool isResponseSuccess(int statusCode) {
    bool result = false;
    if (statusCode == 200 ||
        statusCode == 201 ||
        statusCode == 202 ||
        statusCode == 203 ||
        statusCode == 204 ||
        statusCode == 205 ||
        statusCode == 206 ||
        statusCode == 207 ||
        statusCode == 208 ||
        statusCode == 226) result = true;
    return result;
  }

  bool isResponseErrorNetwork(int statusCode) {
    var result = false;
    if (statusCode == 500 ||
        statusCode == 404 ||
        statusCode == 301 ||
        statusCode == 302 ||
        statusCode == 303) result = true;
    return result;
  }

  dynamic responseBody(var response) {
    ///Status Response Success
    if (isResponseSuccess(response.statusCode)) {
      return response.body;
    }

    ///Status Response ErrorNetwork
    else if (isResponseErrorNetwork(response.statusCode)) {
      throw allTranslations.text('mess_network_failed');
    }

    ///Other Server Response mess
    else {
      var error = response.body.toString();
      if (isHasJsonMessage(error) != null) {
        throw JsonMessage.fromJson(jsonDecode(error)).message.toString();
      } else
        throw response.error ?? '';
    }
  }
}

