import 'dart:async';

import 'package:hqsoft_core/core_base/core_base.dart';
import 'package:hqsoft_core/data_base/data_base.dart';


class UserRepository {
  final userApiClient = UserApiClient();

  Future<ResultTokenLogin> authenticateToken({
    required String countryCode,
    required String username,
    required String password,
  }) async {
    return userApiClient.login(countryCode, username, password);
  }

  Future<ResultTokenLogin> getTokenConnect({
    required String countryCode,
    required String username,
    required String password,
  }) async {
    return userApiClient.getTokenConnect(countryCode, username, password);
  }

  Future<UserAppToken> createUserAppTokenFCM({
    required String tokenAuthor,
    required String userId,
    required String appId,
    required String tokenFCM,
  }) async {
    return userApiClient.createUserAppTokenFCM(
        tokenAuthor, userId, appId, tokenFCM);
  }

  Future<bool> updateCreateDateUserAppTokenFCM({
    required String tokenAuthor,
    required int id,
  }) async {
    return userApiClient.updateCreateDateUserAppTokenFCM(
        tokenAuthor, id);
  }

  Future<bool> deleteUserAppTokenFCM({
    required int id,
  }) async {
    return userApiClient.deleteUserAppTokenFCM(id);
  }

  Future<void> deleteToken() async {
    /// delete from keystore/keychain
    await Future.delayed(Duration(seconds: 1));
    return;
  }

  Future<void> saveTokenLoginAuthor(ResultTokenLogin resultTokenLogin) async {
    /// write to keystore/keychain
    await tokenPreferences.setPreferredResultTokenLogin(resultTokenLogin);
    return;
  }

  Future<void> persistToken(ResultTokenLogin resultTokenLogin,
      bool isSettingFingerPrint, int idTokenFCM, {bool isWeb=false}) async {
    /// refresh setting Biometrics = false
    bool _isSettingFingerPrint = isSettingFingerPrint ?? false;

    ResultTokenLogin? resultTokenLoginOld =
        await tokenPreferences.getPreferredResultToken();

    if(!isWeb) {
      if (resultTokenLoginOld == null ||
          (resultTokenLoginOld.identity != resultTokenLogin.identity))
        await tokenPreferences.setPreferredIsBiometrics(false);
      if (_isSettingFingerPrint)
        await tokenPreferences.setPreferredIsBiometrics(true);
    }

    /// set token FireBase Message Cloud
    await tokenPreferences.setPreferredTokenFCM(
        await tokenFireBaseReferences.getPreferredTokenFCM());

    /// set ID Token FCM form service
    if (idTokenFCM != -1)
      await tokenPreferences.setPreferredIdTokenFCM(idTokenFCM);

    /// write to keystore/keychain
    await tokenPreferences.setPreferredResultTokenLogin(resultTokenLogin);

    /// set isLogin= true
    await tokenPreferences.setPreferredIsLogin(true);

//    await Utils.log(
//        "DEVICE", Utils.getTypeDevice(), await Utils.getDeviceDetails());
    return;
  }

  Future<ResultTokenLogin> hasToken() async {
    /// read from keystore/keychain
    ResultTokenLogin? resultTokenLogin =
        await tokenPreferences.getPreferredResultToken();
    if (resultTokenLogin == null) {
      resultTokenLogin = new ResultTokenLogin();
      resultTokenLogin.countryCode =
          await serviceCountryPreferences.getPreferredServiceCountry();
      resultTokenLogin.phone = "";
      resultTokenLogin.identity = "";
      resultTokenLogin.token = "";
      resultTokenLogin.password = '';
    }
    return resultTokenLogin;
  }
}
